; CircuitBread Tutorials
; Microcontroller Basics (PIC10F200), Christmas Lights Special
; Charlieplexing.
; Link: https://www.circuitbread.com/tutorials/christmas-lights-special-microcontroller-basics-pic10f200
; Source code conversion from Microchip MPASM to Microchip XC8 PIC Assembler.

; Include target device specific definitions.
#include <xc.inc>

; Some equates not included in the device specific definitions.
;----- OPTION_REG Bits -----------------------------------------------------
T0CS        EQU 0005h
NOT_GPPU    EQU 0006h
;----- STATUS Bits -----------------------------------------------------
Z           EQU 0002h

; Set the configuration word.
CONFIG WDTE = OFF  ; Watchdog Timer (WDT disabled)
CONFIG CP = OFF    ; Code Protect (Code protection off)
CONFIG MCLRE = OFF ; Master Clear Enable (MCLR disabled, GP3 enabled)

i        EQU    010h   ;define 0x10 register as the delay variable
j        EQU    011h   ;define 0x11 register as the delay variable
k        EQU    012h   ;define 0x12 register as the delay variable
led      EQU    013h   ;define 0x13 register as the LED number

; NOTE: To make sure the PIC10F200 RC oscillator calibration instruction at
; the 0xFF program memory address is not overwriten place this psect elsewhere
; , for example at the program memory start address: Project Properties >
; pic-as Global Options > Additional Options: -Wl,-pMyCode=0h
    PSECT MyCode,class=CODE,delta=2
INIT:
    MOVLW  ~(1<<T0CS)       ;Enable GPIO2
    OPTION    
    MOVLW ((1 << GPIO_GP0_POSITION)|(1 << GPIO_GP1_POSITION)|(1 << GPIO_GP2_POSITION)) ;set GP0, GP1, GP2 as inputs
    TRIS GPIO
LOOP:
    MOVLW 1             ;Light up LED1
    CALL LIGHT_LED
    MOVLW 2             ;Light up LED2
    CALL LIGHT_LED
    MOVLW 3             ;Light up LED3
    CALL LIGHT_LED
    MOVLW 4             ;Light up LED4
    CALL LIGHT_LED
    MOVLW 5             ;Light up LED5
    CALL LIGHT_LED
    MOVLW 6             ;Light up LED6
    CALL LIGHT_LED
    MOVLW 5             ;Light up LED5
    CALL LIGHT_LED
    MOVLW 4             ;Light up LED4
    CALL LIGHT_LED
    MOVLW 3             ;Light up LED3
    CALL LIGHT_LED
    MOVLW 2             ;Light up LED2
    CALL LIGHT_LED
    GOTO LOOP           ;loop forever

DELAY:                  ;Start DELAY subroutine here
    MOVLW 2             ;Load initial value for the delay
    MOVWF i             ;Copy the value to the register 0x10
    MOVWF j             ;Copy the value to the register 0x11
    MOVWF k             ;Copy the value to the register 0x12
DELAY_LOOP:             ;Start delay loop
    DECFSZ i, F         ;Decrement the register i and check if not zero
    GOTO DELAY_LOOP     ;If not then go to the DELAY_LOOP label
    DECFSZ j, F         ;Else decrement the register j, check if it is not 0
    GOTO DELAY_LOOP     ;If not then go to the DELAY_LOOP label
    DECFSZ k, F         ;Else decrement the register k, check if it is not 0
    GOTO DELAY_LOOP     ;If not then go to the DELAY_LOOP label
    RETLW 0             ;Else return from the subroutine

SELECT_LED:             ;Turn on LED1
    DECFSZ led, F       ;Decrement the register 'led' and check if not zero
    GOTO LED2           ;If not then go to the LED2 label
    MOVLW ~((1 << GPIO_GP0_POSITION)|(1 << GPIO_GP1_POSITION))
    TRIS GPIO           ;Otherwise set GP0 and GP1 as outputs
    MOVLW 1 << GPIO_GP0_POSITION      ;Set GP0 pin as output high
    MOVWF GPIO
    RETLW 0             ;and return from the subroutine
LED2:
    DECFSZ led, F       ;Decrement the register 'led' and check if not zero
    GOTO LED3           ;If not then go to the LED3 label    
    MOVLW ~((1 << GPIO_GP0_POSITION)|(1 << GPIO_GP1_POSITION))
    TRIS GPIO           ;Otherwise set GP0 and GP1 as outputs
    MOVLW 1 << GPIO_GP1_POSITION      ;Set GP1 pin as output high    
    MOVWF GPIO
    RETLW 0             ;and return from the subroutine
LED3:
    DECFSZ led, F       ;Decrement the register 'led' and check if not zero
    GOTO LED4           ;If not then go to the LED4 label    
    MOVLW ~((1 << GPIO_GP1_POSITION)|(1 << GPIO_GP2_POSITION))
    TRIS GPIO           ;Otherwise set GP1 and GP2 as outputs
    MOVLW 1 << GPIO_GP1_POSITION      ;Set GP1 pin as output high
    MOVWF GPIO
    RETLW 0             ;and return from the subroutine
LED4:
    DECFSZ led, F       ;Decrement the register 'led' and check if not zero
    GOTO LED5           ;If not then go to the LED5 label    
    MOVLW ~((1 << GPIO_GP1_POSITION)|(1 << GPIO_GP2_POSITION))
    TRIS GPIO           ;Otherwise set GP1 and GP2 as outputs
    MOVLW 1 << GPIO_GP2_POSITION      ;Set GP2 pin as output high
    MOVWF GPIO
    RETLW 0             ;and return from the subroutine
LED5:
    DECFSZ led, F       ;Decrement the register 'led' and check if not zero
    GOTO LED6           ;If not then go to the LED6 label    
    MOVLW ~((1 << GPIO_GP0_POSITION)|(1 << GPIO_GP2_POSITION))
    TRIS GPIO           ;Otherwise set GP0 and GP2 as outputs
    MOVLW 1 << GPIO_GP0_POSITION      ;Set GP0 pin as output high
    MOVWF GPIO
    RETLW 0             ;and return from the subroutine
LED6:
    DECFSZ led, F       ;Decrement the register 'led' and check if not zero
    RETLW 0             ;If not return from the subroutine    
    MOVLW ~((1 << GPIO_GP0_POSITION)|(1 << GPIO_GP2_POSITION))
    TRIS GPIO           ;Otherwise set GP0 and GP2 as outputs
    MOVLW 1 << GPIO_GP2_POSITION      ;Set GP2 pin as output high
    MOVWF GPIO
    RETLW 0             ;and return from the subroutine

LIGHT_LED:              ;Light one LED and perform delay
    MOVWF led           ;Copy the content of the W into 'led' register
    CALL SELECT_LED     ;Call SELECT_LED subroutine
    CALL DELAY          ;Call DELAY subroutine
    RETLW 0             ;Return from the subroutine

    END INIT        ; Program entry point.